/*
  Library for Teensy 3.6/Teensy 3.2 6.302 board
  Created by teaching staff of 6.302, a control class at MIT.
*/

#ifndef lib6302_h
#define lib6302_h

#include <ADC.h>

#define nSleepPWM 2  // Turns Both PWM chips on and off.
// Dual Hbridge (A and B) with Crnt and Volt Sense
#define pwmA1 22
#define pwmA2 23
#define pwmB1 9
#define pwmB2 10

// Second Dual Hbridge (A_T and B_T) (no crnt sensing)
#define pwmA1_T 3
#define pwmA2_T 4
#define pwmB1_T 5
#define pwmB2_T 6

// Hbridge Crnt and Volt Sense Inputs 
#define IsenseB A7
#define IsenseA A6
#define VsenseA A5
#define VsenseB A4

// DACs
#define DAC1 A22  // A21,A22 for Teensy 3.6.
#define DAC0 A21

// Loop monitor Pin (for testing)
#define loopMonitor 24

// End Pin definitions.

// Definitions for ADC
#define LIB6302_ADC(BUF)\
ADC *_adc = new ADC(); \
volatile uint16_t _bufMax = BUF;\
volatile uint16_t _numSamples = 0;\
volatile uint16_t _adcArray[BUF+10];\
volatile uint16_t _numS = 0;\

#define MAXPLOT 10
class lib6302
{
 public:
  lib6302(int pwmF, int dacRes, int adcRes, Stream *serialChan);
  void setup(String config_message, float togglePeriod);
  void startPWM(boolean parallelDrive);
  float hbridgeWrite(float normV, int pin);
  float hbridgeBipolar(float normV, int pinP, int pinN);
  void sendStatus(int numStats, float stats[]);
  void startloop();
  bool isFirst();
  void setFirst(boolean val);
  float getFlip();
  float getSlider(int whichVal);
  float getAdcMax();
  float getDacMax();
  void setupADC(int numAvg, boolean goFast);
  float adcGetAvg(const uint8_t pin);
  float adcGetAvg(const uint8_t pin, uint16_t &numSamples);
  float adcGetBuf(const uint8_t pin, uint16_t &numSamples, float sampBuf[]);
 private:
  void _processRcvString(String inStr, float rcvBuf[]);
  void _parseConfigStr(String inStr);
  int _adcReadIn(const uint8_t pin, const uint16_t numSamples);
    float _rcvBuf[10];
  char _sendBuf[100];
  int _dacRes;
  int _adcRes;
  float _dacMax;
  float _adcMax;
  int _PWMF;
  Stream *_serChan;
  int _baud;
  float _flipSign = 1.0;
  boolean _firstTime = true;
  boolean _firstStatus = true;
  int _togglePeriod;
  elapsedMicros _sinceToggle = 0;
  boolean _monitorSwitch = false;
  String _config_message = "";
  String _inputString = "";
  int _numPlots = 0;
  float _plotOffsets[MAXPLOT];
  float _plotMins[MAXPLOT];
  float _plotMaxs[MAXPLOT];
  String _plotLabels[MAXPLOT];
};

#endif
